==> distfun-0.3-1-src.zip: distfun/sci_gateway/cgrand/sci_distfun_grandbin.c <==

/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"

// mygenbin --
//
// Generate a random variable from Binomial distribution.
//
// Arguments
// N  (input): the number of trials
// p (input): the probability of success of each trial
// R (output): the number of successes
//
// Note
// Encapsulates R=ignbin(N,p), so that it has the calling sequence 
// required by distfun_computeRandgenA.
// There are two differences
// * N is a double
// * the output R is a double
// We assume that N has an integer value.
// (This is checked earlier in the calling sequence.)
double mygenbin (double * N, double * p)
{
	int intN;
	double R;
	int intR;
	intN = (int) (*N);
	intR = C2F(ignbin)(&intN,p);
	R= (double) intR;
	return R;
}

/**************************************************
Y=distfun_grandbin(m,n,N,p) 

generates random variates from the binomial distribution with 
parameters N (positive integer) and p (real in [0,1]) : 
number of successes in N independant Bernouilli trials with 
probability p of success. 

Related function(s) : binomial, cdfbin.
***********************************************************************/

int sci_distfun_grandbin(char *fname,unsigned long fname_len)
{
	int rRows,rCols;
	int mN, nN;
	int mp, np;
	int readFlag;
	double * pN = NULL;
	double * pp = NULL;

	CheckRhs(4,4);
	CheckLhs(1,1);

	// Get Arg #1 = M, Arg #2 = N
	readFlag = distfun_GetMN ( fname , &rRows , &rCols );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Arg #3 : N
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 3, &pN, &mN, &nN);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixHasIntegerValue ( fname, 3, pN, mN , nN );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixGreaterOrEqual (fname, 3, pN, mN, nN, 0.0);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Arg #4 : p
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 4, &pp, &mp, &np);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixInRange (fname, 4, pp, mp, np, 0.0, 1.0);
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Compute the result (if necessary, expand the arguments N and p)
	readFlag = distfun_computeRandgenAB (fname, mN, nN, pN, mp, np, pp, rRows, rCols, mygenbin );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	LhsVar(1) = Rhs+1;

	return 0;
}

==> distfun-0.5-1-src.zip: distfun/sci_gateway/cgrand/sci_distfun_grandbin.c <==

/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"

// mygenbin --
//
// Generate a random variable from Binomial distribution.
//
// Arguments
// N  (input): the number of trials
// p (input): the probability of success of each trial
// R (output): the number of successes
//
// Note
// Encapsulates R=ignbin(N,p), so that it has the calling sequence 
// required by distfun_computeRandgenA.
// There are two differences
// * N is a double
// * the output R is a double
// We assume that N has an integer value.
// (This is checked earlier in the calling sequence.)
double mygenbin (double N, double p)
{
	int intN;
	double R;
	int intR;
	intN = (int) N;
	intR = genrand_ignbin(intN,p);
	R= (double) intR;
	return R;
}

/**************************************************
Y=distfun_grandbin(m,n,N,p) 

generates random variates from the binomial distribution with 
parameters N (positive integer) and p (real in [0,1]) : 
number of successes in N independant Bernouilli trials with 
probability p of success. 

Related function(s) : binomial, cdfbin.
***********************************************************************/

int sci_distfun_grandbin(char *fname,unsigned long fname_len)
{
	int rRows,rCols;
	int mN, nN;
	int mp, np;
	int readFlag;
	double * pN = NULL;
	double * pp = NULL;

	CheckInputArgument(pvApiCtx,4,4);
	CheckOutputArgument(pvApiCtx,1,1);

	// Get Arg #1 = M, Arg #2 = N
	readFlag = distfun_GetMN ( fname , &rRows , &rCols );
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	// Arg #3 : N
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 3, &pN, &mN, &nN);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixHasIntegerValue ( fname, 3, pN, mN , nN );
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixGreaterOrEqual (fname, 3, pN, mN, nN, 0.0);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	// Arg #4 : p
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 4, &pp, &mp, &np);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixInRange (fname, 4, pp, mp, np, 0.0, 1.0);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
    // Compute the result (if necessary, expand the arguments N and p)
	readFlag = distfun_computeRandgenAB (fname, mN, nN, pN, mp, np, pp, rRows, rCols, mygenbin );
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	LhsVar(1) = Rhs+1;

	return 0;
}
